c This is a slightly modified version of Pat Keast's program for 
c testing quadrature routines.
c 
c It is initialized to test the TEAPACK routine DAQUAD based on the
c two-point Gauss-Legendre rule.  To test your modified version of
c DAQUAD, change the statements contained between comment lines of
c the form
c 
c***********************************************************************
c      instructions for changing statements
c      statements to be changed
c***********************************************************************
c 
c in the main program, block data subprogram and function subprogram
c apint immediately below.  You should not need to change any other
c segments of this test program.
c 
c I have made one major change to the test program by adding
c     common/krj/nfcn
c to each test function F1,...,F7 and incrementing nfcn by one each
c time each function is called.  This provides an easy way of setting
c NFCALL (the number of function calls used to solve the problem)
c in apint.
c
c I have also added a lot of missing declarations and corrected a few
c typos.  I have made a few other minor modification to the program.  
c The more important ones are proceeded by a comment contained between
c lines of *'s of the form described above.
c 
c All of my added comments, statements, and modifications are in small
c letters; the original program is in capitals.
c 
c To run this test program, execute the command
c    f77 test.f daquad.f dqud.f dglegq.f
c Ignore the warning from f77 about
c        "character datum assigned to an integer variable".
c All output is to unit 6.  On Unix systems, this is the standard output
c (i.e., it is printed to the screen or can be redirected to a file).
c You may find it helpful to filter the output through fpr before you 
c look at it  (i.e.,  a.out | fpr > io.file ).
c
c It is NOT necessary to run this test program on DAQUAD;  I have already
c done so and left the results in ~krj/quadrature/test.dat for you to
c examine.  Also, since I already have copy of test.dat, you need NOT 
c hand-in a copy of test.dat with your own test results, even though you 
c probably will refer to it in your comparison of DAQUAD and your modified 
c routine.
c 
c I have found a few instances of discrepancies between what Keast's
c Tech Report says the test program does and what it actually does.  In
c particular, the definitions of the functions on pp. A1-2 do not agree
c completely with what is in this program.  Remind me to mention this in
c our next class.  Also, I have made a few modifications to the test program
c as indicated above.
c
c Report any problems with test.f to me by e-mail to krj.
c 
c K.R.J.  11/11/93



c Main driver program for Keast's quadrature test routine.

      double precision apint
      integer mgroup(7), mtol(7,2), ng, i
      EXTERNAL APINT

      NG = 7

      DO 1 I = 1,7
         MTOL(I,1) = 1
         MTOL(I,2) = 10
         MGROUP(I) = 1
 1    continue

c***********************************************************************
c
c The following 7 statements should be commented out in your final run
c so that your routine will be tested on the full set of problems on
c the full range of tolerances.  However, during your initial attempts
c to get the routine running, leave these statements active so that
c the your routine will be tested on the 2nd group of problems only
c and on a reduced range of tolerances, thus saving a LOT of computer
c time.
c
      mtol(2,2) = 5
      MGROUP(1) = 0 
      MGROUP(3) = 0 
      MGROUP(4) = 0 
      MGROUP(5) = 0 
      MGROUP(6) = 0 
      mgroup(7) = 0 
c
c***********************************************************************

      CALL TEST (MGROUP,MTOL,APINT,NG)

      STOP
      END



      BLOCK DATA

      DOUBLE PRECISION RERR
      integer kount,nmin,level,iflag,nl1,nl2,maxtrp
      COMMON/QPAR/RERR,KOUNT,NMIN,LEVEL,IFLAG,NL1,NL2
      COMMON/QPAR1/MAXTRP

C     QPAR CONTAINS PARAMETERS USED BY THE QUADRATURE RULE. THESE ARE  C
C              NMIN : A MINIMUM NUMBER OF FUNCTION EVALUATIONS USED    C
C                     BY THE QUADRATURE ROUTINE UNDER EXAMINATION.     C
C           NL1,NL2 : USED TO STORE THE NAME OF THE QUADRATURE ROUTINE C
C                     WHICH MUST BE LESS THAN 8 CHARACTERS             C
C             LEVEL : A FREE PARAMETER USED IN DIFFERENT WAYS IN       C
C                     DIFFERENT ROUTINES. E.G. IT IS THE MAXIMUM       C
C                     NUMBER OF EXTRAPOLATIONS IN THE ROMBERG METHODS. C
C              RERR : A FREE PARAMETER TO BE USED IF REQUIRED.         C
C     IF THE QUADRATURE ROUTINE REQUIRES MORE DATA, INSERT A LABELLED  C
C     COMMON HERE AND IN THE INTEGRATION METHOD.                       C

C***********************************************************************
c
c     change DAQUAD to the name of your routine.  Note:  divide the name
c     of your routine into two groups of four letters each as has been
c     done for 'DAQU' 'AD  ' below.  
c     You may also have to change LEVEL if you have changed MAXLEV in 
c     your modification of DAQUAD.

      DATA NMIN, LEVEL, NL1, NL2 / 6, 30, 'DAQU', 'AD  ' /

C***********************************************************************

      END 


      double precision function apint (f,a,b,tol,errest)

cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c                                                                      c
c This routine is of the form required by Keast's quadrature test      c
c program.  It calls the quadrature routine that you are actually      c
c testing and acts as an interface between it and the test program.    c
c Its parameters are:                                                  c
c   f      : is an external function                                   c
c   a,b    : are the limits of integration                             c
c   tol    : the requested tolerance, not changed by apint             c
c            (unlike daquad)                                           c
c   errest : is the estimated absolute error                           c
c It also sets NFCALL passed through the common block QPAR.            c
c                                                                      c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      double precision f, a,b, tol, tol1, errest, sing, daquad
      integer maxlev
      double precision rerr
      integer nfcall,nmin,level,iflag,nl1,nl2
      integer nfcn
      external f, daquad, dglegq
      common/qpar/rerr,nfcall,nmin,level,iflag,nl1,nl2
      common/krj/nfcn

      maxlev = level
      nfcn = 0

c***********************************************************************
c
c If the arguments for your modified version of daquad are the same as 
c those for daquad, then you need only change the name of daquad to 
c the name of your modified routine to test the latter.

      tol1 = tol
      apint = daquad(dglegq,2,f,a,b,tol1,maxlev,sing,errest)

c***********************************************************************

      nfcall = nfcn
   
      if (tol1 .ne. 0.d0) write (6,1) tol1, sing
   1  format(/ ' ******   Warning from apint:   tol =', d10.3,
     +          ',  sing =', d10.3, '   ******' / )

      return
      end




      SUBROUTINE TEST (MGROUP,MTOL,APINT,NG)

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                                                  C
C   MGROUP IS A VECTOR OF NG COMPONENTS, WHERE NG IS THE NUMBER    C
C OF GROUPS OF INTEGRANDS. IF MGROUP(I) = 1, THE ITH GROUP IS      C
C CALLED, OTHERWISE IT IS NOT USED.  MTOL IS A TWO DIMENSIONAL     C
C NG BY 2 MATRIX INDICATING THE TOLERANCES TO BE USED IN EACH      C
C GROUP. IF MTOL(I,1) = K AND MTOL(I,2) = L, (K<=L), THEN          C
C TOLERANCES 10**(-K) TO 10**(-L) ARE USED. THE DEFAULT OPTION     C
C CALLS ALL GROUPS, AND USES TOLERANCES 10**(-1) TO 10**(-10), AND C
C IS OBTAINED BY PUTTING MGROUP(1) = -1, AND MTOL(1,1) = -1.       C
C      APINT IS THE QUADRATURE METHOD, WHICH MUST BE IN THE FORM   C
C OF A FUNCTION SUBPROGRAM, WITH THE CALLING SEQUENCE              C
C          APPROX = APINT(F,A,B,TOL,ERR)                           C
C  WHERE   APPROX : IS THE APPROXIMATE INTEGRAL                    C
C          F      : IS AN EXTERNAL FUNCTION                        C
C          A,B    : ARE THE LIMITS OF INTEGRATION                  C
C          ERR    : IS THE ESTIMATED ABSOLUTE ERROR                C
C                                                                  C
C      TO INSERT NEW GROUPS SEE WRITE-UP.                          C
C                                                                  C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

      DOUBLE PRECISION DFLOAT
      DOUBLE PRECISION F1,F2,F3,F4,F5,F6,F7,ANSWR,APINT,Z
      DOUBLE PRECISION ALPH1(30),BETA1(30)
      integer MGROUP(NG),MTOL(NG,2),MM(30),KK(30),JJ(30)
      integer i, j, ij, ng, n, l1, l2
      EXTERNAL F1,F2,F3,F4,F5,F6,F7
      EXTERNAL ANSWR
      EXTERNAL APINT
        
C     CHECK FOR VALIDITY OF MTOL.

      DO 1111 I = 1,NG
        IF (MTOL(I,2) .GE. MTOL(I,1)) GO TO 1111
        MTOL(I,2) = MTOL(I,1)
        WRITE(6,1112) I
 1112   FORMAT (' ','MTOL(',I2,') IS NOT VALID. LARGER TOLERANCE TAKEN')
 1111 CONTINUE

      IF (MGROUP(1) .NE. -1) GO TO 100

C   IF THE DEFAULT VALUES OF MTOL AND MGROUP ASKED FOR,
C   THEY ARE GENERATED NOW.

      DO 1113 I = 1,NG
 1113    MGROUP(I) = 1

 100  IF (MTOL(1,1) .NE. -1) GO TO 101

      DO 1114 I = 1,NG
         MTOL(I,1) = 1
 1114    MTOL(I,2) = 10

101   CONTINUE

C     INITIALISE GROUP PARAMETERS.  THE VECTORS JJ,MM,KK, ETC., SPECIFY
C     WHICH FUNCTIONS IN EACH GROUP ARE USED.  THESE MAY BE ALTERED BY
C     CHANGING THE DATA AND ASSIGNMENT STATEMENTS PRECEDING EACH CALL
C     OF TSTGRP.  THE TEST PROGRAM FOR EACH GROUP.
C     FOR AN EXPLANATION OF THE PARAMETERS IN THE CALLS OF TSTGRP,
C     SEE TSTGRP.

      DO 1 I = 1,30
         JJ(I) = 0
         KK(I) = 0
         MM(I) = 0
         BETA1(I) = 0.D0
1        ALPH1(I) = 0.D0

      IF (MGROUP(1) .NE. 1) GO TO 200

C     THE PARAMETERS FOR GROUP 1 ARE NOW CALCULATED

      DO 10 I = 1,20
         KK(I) = I
10       JJ(I) = 1

      DO 11 I = 21,26
11       JJ(I) = I-19

      N = 26
      L1 = MTOL(1,1)
      L2 = MTOL(1,2)

      CALL TSTGRP (F1,1,ANSWR,JJ,KK,MM,ALPH1,BETA1,N,L1,L2,APINT)

C     END OF GROUP 1

200   CONTINUE

      IF (MGROUP(2) .NE. 1) GO TO 300

C     THE PARAMETERS FOR GROUP 2 ARE NOW CALCULATED

      JJ(1) = 1
      JJ(2) = 2

      DO 20 I = 1,4
         JJ(I+2) = 3
         JJ(I+6) = 4
         MM(I+2) = I
20       MM(I+6) = I

      N = 10
      L1 = MTOL(2,1)
      L2 = MTOL(2,2)

      CALL TSTGRP (F2,2,ANSWR,JJ,KK,MM,ALPH1,BETA1,N,L1,L2,APINT)

C     END OF GROUP 2

300   CONTINUE

      IF (MGROUP(3) .NE. 1) GO TO 400

C     THE PARAMETERS FOR GROUP 3 ARE NOW CALCULATED.

      DO 30 I = 1,2
         Z = 1.D0/DFLOAT(I+1)
         DO 31 J = 1,8
            IJ = J + (I-1)*8
            ALPH1(IJ) = 1.D0/DFLOAT(2**J)
 31         BETA1(IJ) = Z
 30   CONTINUE

      N = 16
      L1 = MTOL(3,1)
      L2 = MTOL(3,2)

      CALL TSTGRP (F3,3,ANSWR,JJ,KK,MM,ALPH1,BETA1,N,L1,L2,APINT)

C     END OF GROUP 3.

 400  CONTINUE

      IF (MGROUP(4) .NE. 1) GO TO 500

C     THE PARAMETERS FOR GROUP 4 ARE NOW CALCULATED.

      DO 40 I = 1,5
         Z = DFLOAT(6-I)*.1D0
         DO 41 J = 1,4
            IJ = I + (J-1)*5
            JJ(IJ) = J
            ALPH1(IJ) = Z
 41         BETA1(IJ) = 1.D0/2.D0
         DO 42 J = 5,6
            IJ = I + (J-1)*5
            JJ(IJ) = 2*(J-4)
            ALPH1(IJ) = Z
 42         BETA1(IJ) = 1.D0/3.D0
 40   CONTINUE

      N = 30
      L1 = MTOL(4,1)
      L2 = MTOL(4,2)

      CALL TSTGRP (F4,4,ANSWR,JJ,KK,MM,ALPH1,BETA1,N,L1,L2,APINT)

C     END OF GROUP 4.

 500  CONTINUE

      IF (MGROUP(5) .NE. 1) GO TO 600

C     THE PARAMETERS FOR GROUP 5 ARE NOW CALCULATED.

      DO 50 I = 1,5
         Z = .5D0 + DFLOAT(I)*.5D-1
         DO 51 J = 1,4
            IJ = I + (J-1)*5
            JJ(IJ) = J
            ALPH1(IJ) = -Z
            BETA1(IJ) = 1.D0/2.D0
 51      CONTINUE
         DO 52 J = 5,6
            IJ = I + (J-1)*5
            JJ(IJ) = 2*(J-4)
            ALPH1(IJ) = -Z
 52         BETA1(IJ) = 1.D0/3.D0
 50   CONTINUE

      N = 30
      L1 = MTOL(5,1)
      L2 = MTOL(5,2)

      CALL TSTGRP (F5,5,ANSWR,JJ,KK,MM,ALPH1,BETA1,N,L1,L2,APINT)

C     END OF GROUP 5.

  600 CONTINUE

      IF (MGROUP(6) .NE. 1) GO TO 700

C     THE PARAMETERS FOR GROUP 6 ARE NOW CALCULATED

      DO 60 I = 1,4
         DO 61 J = 1,5
            IJ = I+(J-1)*4
            KK(IJ) = I
            MM(IJ) = J*5
   61    CONTINUE
   60 CONTINUE

      N = 20
      L1 = MTOL(6,1)
      L2 = MTOL(6,2)

      CALL TSTGRP (F6,6,ANSWR,JJ,KK,MM,ALPH1,BETA1,N,L1,L2,APINT)

C     END OF GROUP 6.

  700 CONTINUE

      IF (MGROUP(7) .NE. 1) GO TO 800

C     THE PARAMETERS FOR GROUP 7 ARE NOW CALCULATED

      DO 70 I = 1,4
         DO 71 J = 1,5
            IJ = I+(J-1)*4
            MM(IJ) = J*5
            KK(IJ) = I
   71    CONTINUE
   70 CONTINUE

      N = 20
      L1 = MTOL(7,1)
      L2 = MTOL(7,2)

      CALL TSTGRP (F7,7,ANSWR,JJ,KK,MM,ALPH1,BETA1,N,L1,L2,APINT)

C     END OF GROUP 7.

  800 CONTINUE

      RETURN
      END



      SUBROUTINE TSTGRP (F,NOFG,ANS,JJ,KK,MM,ALPH1,BETA1,N,L1,L2,APINT)

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                                                      C
C  F SPECIFIES WHICH GROUP OF FUNCTIONS IS BEING CALLED, ANS THE       C
C  ANSWER GROUP, N THE TOTAL NUMBER OF INTEGRATIONS IN THE GROUP.      C
C  JJ, KK, MM, ALPH1 ARE PARAMETERS SPECIFYING THE FUNCTIONS IN EACH   C
C  GROUP.  THEY ARE DEFINED IN THE SUBROUTINE TEST, IN DATA AND        C
C  ASSIGNMENTS PRECEDING THE CALLS TO TSTGRP.  L1, L2 ARE THE UPPER    C
C  AND LOWER BOUNDS OF THE TOLERANCES USED.                            C
C      RERR:  THE ESTIMATED ERROR FROM THE QUADRATURE ROUTINE.         C
C    NFCALL:  THE NUMBER OF FUNCTION CALLS.                            C
C  SUBROUTINE QVAL MEASURES THE QUALITY OF THE RESULTS.  THE           C
C  SUCCESS OR FAILURE OF THE ROUTINE IS ALSO CALCULATED IN QVAL.       C
C                                                                      C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

      DOUBLE PRECISION DATAN, DABS, DFLOAT
      DOUBLE PRECISION APINT, F,ANS,RERR,PI,ALPHA,BETA,ALPH1(N),BETA1(N)
      DOUBLE PRECISION TOL, APPROX, R, EXACT, ERR
      DOUBLE PRECISION AVQ(20), Q
      double precision ave(20), AVQTOT, AVET, e
      integer JJ(N), KK(N), MM(N)
      integer NFN(20), NSUC(20), NFAILS(20), NPSUC(20)
      integer nfcall,nmin,level,iflag,nl1,nl2,maxtrp
      integer nofg, n, l1, l2, icount, i, j, k, l, m, kount
      integer NINTS, NFAILT, NPSUCT, NSUCT
      EXTERNAL F, APINT
      COMMON/QPAR/RERR, NFCALL, NMIN, LEVEL, IFLAG, NL1, NL2
      COMMON/QPAR1/MAXTRP
      COMMON/TESTPR/PI, ALPHA, BETA, K, M, J

      PI = DATAN(1.D0)*4.D0

c     FIRST THE MAIN GROUP HEADING IS OUTPUT


      CALL GRPHED (NOFG,1)

      ICOUNT = 0

      DO 10 I = L1, L2
         AVQ(I) = 0.D0
         AVE(I) = 0.D0
         NFN(I) = 0
         NPSUC(I) = 0
         NSUC(I) = 0
 10      NFAILS(I) = 0

      DO 1 I = 1, N
         J = JJ(I)
         K = KK(I)
         M = MM(I)
         ALPHA = ALPH1(I)
         BETA = BETA1(I)

C        NOTE THAT NOT ALL THESE PARAMETERS ARE USED IN ANY ONE GROUP.

         CALL HEAD (ALPHA,BETA,K,M,J,NOFG)

         EXACT = ANS(NOFG)
         WRITE(6,4) EXACT
    4    FORMAT('+',39X,'ANSWER = ',D17.10,',')
         WRITE(6,6)
    6    FORMAT('0',2X,'TOLERANCE',3X,'ACTUAL ERROR',3X,'EST. ERROR',
     *          3X,'QUALITY',3X,'FN. CALLS',2X,'EFFICIENCY')
         DO 2 L = L1,L2
            TOL = 10.0D0**(-L)
            APPROX = APINT(F,0.D0,1.D0,TOL,ERR)
            R = DABS(EXACT-APPROX)
            CALL QVAL (ERR,R,TOL,Q,KOUNT)
            NFN(L) = NFN(L) + NFCALL
            AVQ(L) = AVQ(L) + Q
            E = 0.D0
            IF (KOUNT-2) 1000,2000,3000
 1000          NSUC(L) = NSUC(L) + 1
               E = DFLOAT(NMIN)/(DFLOAT(NFCALL)*Q)
               AVE(L) = AVE(L) + E
               GO TO 3

 2000          NPSUC(L) = NPSUC(L) + 1
               GO TO 3

 3000          NFAILS(L) = NFAILS(L) + 1
    3       CONTINUE

            WRITE(6,5) L,R,ERR,Q,NFCALL,E
    5       FORMAT(' ',3X,'10**(-',I2,')',3X,D9.2,5X,D9.2,4X,G8.2,3X,I5,
     *             4X,D9.2)
    2    CONTINUE

         ICOUNT = ICOUNT + 1
         IF ( (ICOUNT/4)*4 .EQ. ICOUNT ) CALL GRPHED (NOFG,1)

    1 CONTINUE

      NINTS = (L2-L1+1)*N
      NFAILT = 0
      AVQTOT = 0.D0
      NPSUCT = 0
      NSUCT = 0
      AVET = 0.D0

      DO 20 I = L1,L2
         NFAILT = NFAILT + NFAILS(I)
         AVQTOT = AVQTOT + AVQ(I)
         NPSUCT = NPSUCT + NPSUC(I)
         NSUCT = NSUCT + NSUC(I)
         NFN(I) = NFN(I)/N
         IF (NSUC(I) .EQ. 0) GO TO 71
            AVET = AVET + AVE(I)
            AVE(I) = AVE(I)/DFLOAT(NSUC(I))
   71    CONTINUE
   20    AVQ(I) = AVQ(I)/N

      AVQTOT = AVQTOT/DFLOAT(NSUCT)
      AVET = AVET/DFLOAT(NSUCT)
      CALL GRPHED (NOFG,2)

      WRITE(6,100) NSUCT,NPSUCT,NFAILT,AVQTOT,AVET
  100 FORMAT('0','THE TOTAL NUMBER OF SUCCESSES IS ',I4/' THE TOTAL  NUM
     *BER OF PARTIAL SUCCESSES IS ',I4/'THE TOTAL NUMBER OF FAILURES IS'
     *,I4/' THE AVERAGE Q IS ',G8.2/' THE AVERAGE EFFICIENCY IS ',G8.2)

      WRITE(6,1001)
 1001 FORMAT('0',34X,'PARTIAL')

      WRITE(6,101)
101   FORMAT(' ','TOLERANCE  AVERAGE Q  FAILURES  SUCCESSES  SUCCESSES
     *FN.CALLS  EFFICIENCY')

      DO 200 J = L1,L2
         WRITE(6,201)J,AVQ(J),NFAILS(J),NPSUC(J),NSUC(J),NFN(J),AVE(J)
201      FORMAT(' ','10**-',I2,4X,G8.2,4X,I4,7X,I4,5X,I4,6X,I6,6X,G8.2)
200   CONTINUE

      RETURN
      END



      SUBROUTINE QVAL (E,R,T,Q,KOUNT)

C     THIS ROUTINE EVALUATES Q. THE QUALITY OF THE RESULTS, IN TERMS
C     OF THE ESTIMATED ERROR E, ACTUAL ERROR R, AND THE TOLERANCE T.
C     SEE THE REPORT FOR DETAILS.

      DOUBLE PRECISION ALPHA,BETA,B,C,F,Q,E,R,T,RT,ET
      integer kount
      DATA ALPHA,BETA,B,C,F/10.D0,50.D0,10.D0,20.D0,100.D0/

C     TO EVALUATE Q, WE MUST FIRST FIND WHICH REGION OF THE PLANE
C     THE POINT (R/T,E/T) LIES.

      RT = R/T
      ET = E/T

      IF (RT-1.D0) 1,1,2
    1 IF (ET-1.D0) 3,3,4
    3 IF (RT-ET) 5,5,6
    2 IF (ET-1.D0) 7,7,8
    8 IF (ET-RT) 9,10,10

    5 Q = 1.D0 + ALPHA*(ET-RT)
      GO TO 11

    6 Q = 1.D0 + BETA*(RT-ET)
      GO TO 11

    4 Q = 1.D0 + ALPHA - C + C*ET - ALPHA*RT
      GO TO 11

   10 Q = 1.D0 - C - B + C*ET + B*RT
      GO TO 11

    9 Q = 1.D0 - C - B + (C+B-F)*ET + F*RT
      GO TO 11

    7 Q = 1.D0 - C -B + BETA - BETA*ET + F*RT
      GO TO 11

   11 IF (Q .LE. 1.D0 + BETA) GO TO 12
      IF (Q .LE. 1.D0 + 9.D0*F) GO TO 13
         KOUNT = 3
         RETURN

   12    KOUNT = 1
         RETURN

   13    KOUNT = 2
         RETURN

      END



      DOUBLE PRECISION FUNCTION F1(X)

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                                                     C
C     REGULAR, WELL-BEHAVED FUNCTIONS. THE FIRST 20 ARE POLYNOMIALS   C
C     OF DEGREE D, D = 0,1,2,.....19. THE INTERVAL IS 0 <= X <= 1     C
C                                                                     C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

      DOUBLE PRECISION ALPHA,BETA,PI,Z,X
      DOUBLE PRECISION DFLOAT,DEXP,DSIN,DCOS
      double precision y
      integer k,m,j,k1,i
      integer nfcn
      COMMON/TESTPR/PI,ALPHA,BETA,K,M,J

      common/krj/nfcn
      nfcn = nfcn + 1

      GO TO (1,2,3,4,5,6,7),J

 1    F1 = 1.D0
      IF (K .EQ. 1) RETURN
      Z = 1.D0
      K1 = K-1
      DO 10 I = 1,K1
         Z = -Z
 10      F1 = F1*X + Z*DFLOAT(I+1)
      RETURN

 2    F1 = DEXP(X)
      RETURN

 3    F1 = DSIN(PI*X)
      RETURN

 4    F1 = DCOS(X)
      RETURN

 5    F1 = 1.D0/(1.D0 + X*X)
      RETURN

 6    F1 = 1.D0/(1.D0 + DEXP(X))
      RETURN

 7    Y = X*X
      F1 = 1.D0/((Y + 1.D0)*Y + 1.D0)
      RETURN

      END



      DOUBLE PRECISION FUNCTION F2(X)

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                                                     C
C     C-INFINITY FUNCTIONS, WITH A REMOVABLE SINGULARITY. SOME        C
C     OSCILLATORY BEHAVIOR IS ALSO ALLOWED. THE INTERVAL IS           C
C     0 <= X <= 1.                                                    C
C                                                                     C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

      DOUBLE PRECISION ALPHA,BETA,PI,Z,X,Y
      DOUBLE PRECISION DFLOAT,DEXP,DSIN,DCOS,DLOG
      DOUBLE PRECISION P
      integer k,m,j
      integer nfcn
      COMMON/TESTPR/PI,ALPHA,BETA,K,M,J

      common/krj/nfcn
      nfcn = nfcn + 1

      GO TO (1,2,3,4),J

 1    IF (X .EQ. 0.D0) GO TO 11
         Z = 2.D0**X
         F2 = X*Z/(Z-1.D0)
         RETURN

 11      F2 = 1.D0/DLOG(2.D0)
         RETURN

 2    F2 = 1.D0
      IF (X .EQ. 0.D0) RETURN
         F2 = X/(DEXP(X)-1.D0)
         RETURN

 3    IF (X .EQ. 0.D0) GO TO 13
         Y = PI*X
         Z = Y/2.D0
         F2 = DSIN(DFLOAT(M)*Y)*DCOS(Z)/DSIN(Z)
         RETURN

 13      F2 = DFLOAT(M+M)
         RETURN

 4    Z = DFLOAT(M)
      F2 = Z
      IF (X .EQ. 0.D0) RETURN
      IF (X .EQ. 1.D0) GO TO 14
         P = PI*X
         F2 = DSIN(Z*P)/DSIN(P)
         RETURN

 14      IF ( (M/2)*2 .EQ. M ) F2 = -Z
         RETURN

      END



      DOUBLE PRECISION FUNCTION F3(X)

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                                                      C 
C FUNCTIONS WITH A PEAK OF SIZE ALPHA**(-2) AT THE POINT BETA IN THE   C
C RANGE 0 <= X <= 1, WHERE 0 < ALPHA < 1/2.  F3 HAS AT LEAST 1/10TH    C
C OF ITS PEAK VALUE IN (BETA-3*ALPHA, BETA+3*ALPHA), AND ACHIEVES      C
C 9/10TH OF ITS PEAK VALUE IN (BETA-ALPHA/3, BETA+ALPHA/3), THE WIDTH  C
C OF THE PEAK IS THUS ABOUT 2*ALPHA.                                   C
C                                                                      C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

      double precision dflmax
      DOUBLE PRECISION ALPHA, BETA, PI, X, Y, z
      integer k,m,j
      integer nfcn
      COMMON/TESTPR/PI, ALPHA, BETA, K, M, J

      common/krj/nfcn
      nfcn = nfcn + 1

c***********************************************************************
c The following original definition of F3 appears to be wrong for 
c X = BETA.  Therefore, I have change it to agree with Keast's TR.
c Since this changes the integrand at one point only, it does not
c change the value of the integral.
c
c     F3 = 0.D0
c     Y = X - BETA
c
c     IF (Y .EQ. 0.D0) RETURN
c     F3 = 1.D0 / (ALPHA * ALPHA + Y * Y)
c
c Note introduction of declaration for dflmax and z above.  
c Changed version follows.			
c                                  K.R.J.  28/11/88
c***********************************************************************

      Y = X - BETA
      z = alpha * alpha + y * y
      if ( z .eq. 0.d0 ) then
         f3 = .5d0 * dflmax()
      else
         F3 = 1.D0 / (ALPHA * ALPHA + Y * Y)
      endif

      RETURN
      END



      DOUBLE PRECISION FUNCTION F4(X)

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                                                      C
C  FUNCTIONS HAVING DERIVATIVE SINGULARITIES, OR A REMOVABLE           C
C  SINGULARITY AT BETA IN THE RANGE.  THE PARAMETER ALPHA IS > 0, AND  C
C  THE INTERVAL IS 0 <= X <= 1.                                        C
C                                                                      C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

      DOUBLE PRECISION DABS, DLOG
      DOUBLE PRECISION ALPHA, BETA, PI, X, Y
      integer k,m,j
      integer nfcn
      COMMON/TESTPR/PI, ALPHA, BETA, K, M, J

      common/krj/nfcn
      nfcn = nfcn + 1

c***********************************************************************
c The following original definition of F4 appears to be wrong for X = 0 
c or X = BETA.  Therefore, I have change it to agree with Keast's TR.
c Since this changes the integrand at one point only, it does not
c change the value of the integral.
c
c     F4 = 0.D0
c     Y = DABS(X - BETA)
c     
c     IF (X. EQ. 0.D0 .OR. Y .EQ. 0.D0) RETURN
c
c     GO TO (1, 2, 3, 4), J
c
c 1   F4 = X ** ALPHA
c     RETURN
c
c 2   F4 = Y ** ALPHA
c     RETURN
c
c 3   F4 = (X ** ALPHA) * DLOG(X)
c     RETURN
c
c 4   F4 = (Y ** ALPHA) * DLOG(Y)
c     RETURN
c
c Changed version follows.			K.R.J.  28/11/88
c***********************************************************************

      Y = DABS(X - BETA)
      
      GO TO (1, 2, 3, 4), J

  1   continue
      if ( x .eq. 0.d0 ) then
	 f4 = 0.d0
      else
         F4 = X ** ALPHA
      endif
      RETURN

  2   continue
      if ( y .eq. 0.d0 ) then
         f4 = 0.d0
      else
          F4 = Y ** ALPHA
      endif
      RETURN

  3   continue
      if ( x .eq. 0.d0 ) then
         f4 = 0.d0
      else
         F4 = (X ** ALPHA) * DLOG(X)
      endif
      RETURN

  4   continue
      if ( y .eq. 0.d0 ) then
         f4 = 0.d0
      else
         F4 = (Y ** ALPHA) * DLOG(Y)
      endif
      RETURN
   
      END


 

      DOUBLE PRECISION FUNCTION F5(X)

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                                                      C
C  INTEGRABLE FUNCTIONS WITH SINGULARITIES AT BETA IN THE INTERVAL.    C
C  THE PARAMETER ALPHA IS < 0, AND > -1.  THE INTERVAL IS 0 <= X <= 1. C
C                                                                      C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

      DOUBLE PRECISION DABS, DLOG
      DOUBLE PRECISION ALPHA, BETA, PI, X, Y
      integer k,m,j
      integer nfcn
      COMMON/TESTPR/PI, ALPHA, BETA, K, M, J

      common/krj/nfcn
      nfcn = nfcn + 1

      F5 = 0.D0
      
      GO TO (1, 2, 3, 4), J

  1   IF (X .EQ. 0.D0) RETURN
      F5 = X ** ALPHA
      RETURN

  2   Y = DABS(X - BETA)
      IF (Y .EQ. 0.D0) RETURN
      F5 = Y ** ALPHA
      RETURN

  3   IF (X .EQ. 0.D0) RETURN
      F5 = X ** ALPHA * DLOG(X)
      RETURN

  4   Y = DABS(X - BETA)
      IF (Y .EQ. 0.D0) RETURN
      F5 = Y ** ALPHA * DLOG(Y)
      RETURN

      END



      DOUBLE PRECISION FUNCTION F6(X)

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                                                      C
C     PIECEWISE C-INFINITY FUNCTIONS, WITH FINITE JUMPS AT X =         C
C     N/M,  N = 1,2,.....,M-1.                                         C
C     THE PARAMETERS J,M ARE INTEGERS. THE FUNCTIONS ARE PIECEWISE     C
C     POLYNOMIALS OF DEGREE K-1. THE INTERVAL IS 0 <= X <= 1.          C
C                                                                      C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

      DOUBLE PRECISION DFLOAT
      DOUBLE PRECISION ALPHA,BETA,PI,X,Y
      integer k,m,j,m1,k1,i
      integer nfcn
      COMMON/TESTPR/PI,ALPHA,BETA,K,M,J

      common/krj/nfcn
      nfcn = nfcn + 1

      M1 = M*X
      F6 = DFLOAT(M1)

      IF (K .EQ. 1) RETURN

      K1 = K-1
      Y = 1.D0
      DO 1 I = 1,K1
    1    Y = Y*X

      F6 = Y*F6

      RETURN
      END



      DOUBLE PRECISION FUNCTION F7(X)

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                                                      C
C  C-INFINITY FUNCTIONS WHICH OSCILLATE RAPIDLY IN THE RANGE OF        C
C  INTEGRATION. THE NUMBER OF OSCILLATIONS IS M, K-1 IS THE POWER OF   C
C  X BY WHICH THE OSCILLATING FUNCTION IS MULTIPLIED. THE INTERVAL     C
C  IS 0 <= X <= 1.                                                     C
C                                                                      C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

      DOUBLE PRECISION DFLOAT,DSIN
      DOUBLE PRECISION ALPHA,BETA,PI,X,Y,Z
      integer k,m,j,k1,i
      integer nfcn
      COMMON/TESTPR/PI,ALPHA,BETA,K,M,J

      common/krj/nfcn
      nfcn = nfcn + 1

      Z = DFLOAT(M) 
      F7 = DSIN(Z*X*PI)
      IF (K .EQ. 1) RETURN

      Y = 1.D0
      K1 = K-1
      DO 1 I = 1,K1
    1    Y = Y*X
      F7 = Y*F7

      RETURN
      END



      DOUBLE PRECISION FUNCTION ANSWR(N)

      DOUBLE PRECISION DATAN
      DOUBLE PRECISION DFLOAT,DEXP,DSIN,DLOG,DSQRT
      DOUBLE PRECISION ALPHA,BETA,PI,Z,A1,B,B1,B2,W,W2
      double precision a, y, zk, zm
      integer k,m,j
      integer n,m1,k1,nn,i
      COMMON/TESTPR/PI,ALPHA,BETA,K,M,J

      GO TO (1,2,3,4,5,6,7),N

   1  CONTINUE

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                      C
C  ANSWERS FOR INTEGRALS IN GROUP 1    C
C                                      C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

      GO TO (11,12,13,14,15,16,17),J

  11  ANSWR = 1.D0
      IF (K .EQ. 1) RETURN
      Z = 1.D0
      ANSWR = 0.D0
      DO 10 I = 1,K
         ANSWR = ANSWR + Z*DFLOAT(I)/DFLOAT(K-I+1)
  10     Z = -Z
      RETURN

  12  ANSWR = DEXP(1.D0) - 1.D0
      RETURN

  13  ANSWR = 2.D0/PI
      RETURN

  14  ANSWR = DSIN(1.D0)
      RETURN

  15  ANSWR = PI/4.D0
      RETURN

  16  ANSWR = 1.D0 + DLOG(2.D0/(1.D0 + DEXP(1.D0)))
      RETURN

  17  ANSWR = (DLOG(3.D0) + PI/DSQRT(3.D0))/4.D0
      RETURN

   2  CONTINUE

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                     C
C  ANSWERS FOR INTEGRALS IN GROUP 2   C
C                                     C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

      GO TO (21,22,23,24),J

  21  ANSWR = (PI/DLOG(2.D0))**2/12.D0
      RETURN

  22  ANSWR = .7775046341122482D0
      RETURN

  23  ANSWR = 1.D0
      RETURN

  24  IF ( (M/2)*2 .EQ. M ) GO TO 25
         ANSWR = 1.D0
         RETURN
c     else
   25    ANSWR  = 0.D0
         RETURN

    3 CONTINUE

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                      C
C   ANSWERS FOR INTEGRALS IN GROUP 3   C
C                                      C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

      ANSWR = (DATAN((1.D0 - BETA)/ALPHA) + DATAN(BETA/ALPHA))/ALPHA
      RETURN

    4 CONTINUE

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                      C
C   ANSWERS FOR INTEGRALS IN GROUP 4   C 
C                                      C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

      GO TO (43,44,41,42),J

   43 ANSWR = 1.D0/(ALPHA + 1.D0)
      RETURN

   44 A1 = ALPHA + 1.D0
      ANSWR = 1.D0/A1*(BETA**A1 + (1.D0 - BETA)**A1)
      RETURN

   41 ANSWR  = -1.D0/(ALPHA + 1.D0)**2
      RETURN

   42 A1 = ALPHA + 1.D0
      B1 = BETA**A1
      B2 = (1.D0 - BETA)**A1
      ANSWR  = (B1*(DLOG(BETA)-1.D0/A1) + B2*(DLOG(1.D0-BETA) - 1.D0/A1)
     *            )/A1
      RETURN

    5 CONTINUE

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                      C
C   ANSWERS FOR INTEGRALS IN GROUP 5   C
C   THESE ARE THE SAME AS FOR GROUP 4  C
C                                      C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

      GO TO (43,44,41,42),J
      RETURN

    6 CONTINUE

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                      C
C   ANSWERS FOR INTEGRALS IN GROUP 6   C
C                                      C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

      ZK = DFLOAT(K)
      ZM = DFLOAT(M)
      M1 = M - 1
      ANSWR  = ZM - 1.D0
      DO 61 I = 1,M1
   61    ANSWR  = ANSWR  - (DFLOAT(I)/ZM)**K
      ANSWR  = ANSWR / ZK
      RETURN

    7 CONTINUE

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                      C
C   ANSWERS FOR INTEGRALS IN GROUP 7   C
C                                      C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

      Z = 1.D0
      IF ( (M/2)*2 .EQ. M ) Z = -Z
      K1 = K-1
      W = 1.D0/(PI*DFLOAT(M))
      ANSWR = Z*W
      IF (K1 .GT. 0) GO TO 72
         ANSWR  = ANSWR  + W
         RETURN
   72 IF (K1 .EQ. 1) RETURN
      NN = K1/2
      A = 1.D0
      W2 = W*W
      DO 71 I = 1,NN
         Y = DFLOAT(K1-I-I+2)
         A = -A*Y*(Y-1.D0)
         W = W*W2
         B = Z*A*W
   71    ANSWR  = ANSWR  + B
      IF ( (K1/2)*2 .NE. K1 ) RETURN
      ANSWR  = ANSWR  + B*Z
      RETURN
      END


      SUBROUTINE GRPHED (N,M)

      DOUBLE PRECISION RERR
      integer kount,nmin,level,iflag,nl1,nl2,maxtrp
      integer n,m
      COMMON /QPAR/RERR,KOUNT,NMIN,LEVEL,IFLAG,NL1,NL2
      COMMON /QPAR1/MAXTRP

      GO TO (1,2), M

1     WRITE(6,3) N, NL1, NL2
3     FORMAT('1','STATISTICS FOR GROUP ',I2,' FUNCTIONS. METHOD USED IS
     * ',2A4)

c***********************************************************************
c The following statement seems to make no sense, so I've commented 
c it out.
c     NCAL = FNCALL
c***********************************************************************

      RETURN

2     WRITE(6,4) N, NL1, NL2
4     FORMAT('1','GROUP ',I2,' AVERAGE STATISTICS. METHOD USED IS ',2A4)

      RETURN
      END


c*************************************************************************
c The following subroutine declaration seems to be wrong.  It has one more
c parameter than its calling statement.
c     SUBROUTINE HEAD (ALPHA, BETA, K, M, N, J, N)
c     CALL HEAD (ALPHA,BETA,K,M,J,NOFG)
c Therefore, I have changed it to the following:
c*************************************************************************

      SUBROUTINE HEAD (ALPHA, BETA, K, M, J, N)

      DOUBLE PRECISION ALPHA, BETA, ALPHA1, BETA1
      integer k,m,j,n,nk,k1

      ALPHA1 = ALPHA + .1D-06
      BETA1 = BETA + .1D-06
      K1 = K - 1

C     HEADINGS FOR GROUP 1.

      GO TO (1,2,3,4,5,6,7), N
        
1     GO TO (11,12,13,14,15,16,17), J

11    WRITE (6,111) K1
111   FORMAT('0', 'POLYNOMIAL OF DEGREE', I4, '.')
      RETURN

12    WRITE(6,112)
112   FORMAT('0', 'F(X) = EXP(X).')
      RETURN

13    WRITE(6,113)
113   FORMAT('0', 'F(X) = SIN(PI * X).')
      RETURN

14    WRITE(6,114)
114   FORMAT('0', 'F(X) = COS(X).')
      RETURN

15    WRITE(6,115)
115   FORMAT('0', 'F(X) = 1/(1 + X*X).')
      RETURN

16    WRITE(6,116)
116   FORMAT('0', 'F(X) = 1/(1 + EXP(X)).')
      RETURN

17    WRITE(6,117)
117   FORMAT('0', 'F(X) = 1/(X**4 + X**2 + 1).')
      RETURN

C     HEADINGS FOR GROUP 2.

2     GO TO (21,22,23,24), J

21    WRITE(6,121)
121   FORMAT('0', 'F(X) = (2**X) * X / (2**X - 1).')
      RETURN

   22 WRITE(6,122)
  122 FORMAT('0','F(X) = X/(EXP(X) -1).')
      RETURN

   23 WRITE(6,123) M
  123 FORMAT('0','F(X) = SIN(PI*X*',I2,')/TAN(PI*X/2).')
      RETURN

   24 WRITE(6,124) M
  124 FORMAT('0','F(X) = SIN(PI*X*',I2,')/SIN(PI*X).')
      RETURN

C     HEADINGS FOR GROUP 3

    3 ALPHA1 = 1.D0/ALPHA + .1D0
      NK = 2.D0*ALPHA1

      WRITE(6,131) NK,BETA1
  131 FORMAT('0','F(X) = 1/(2**-',I4,'+(X-',F5.2,')**2).')
      RETURN

C     HEADINGS FOR GROUP 4

    4 GO TO (43,44,41,42),J

   41 WRITE(6,141) ALPHA1
  141 FORMAT('0','F(X) = X**',F5.2,'*LN(X).')
      RETURN

   42 WRITE(6,142) BETA1,ALPHA1,BETA1
  142 FORMAT('0','F(X) = |X-',F5.2,'|**',F5.2,'*LN|X-',F5.2,'|.')
      RETURN 

   43 WRITE(6,143) ALPHA1
  143 FORMAT('0','F(X) = X**',F5.2,'.')
      RETURN

   44 WRITE(6,144) BETA1,ALPHA1
  144 FORMAT('0','F(X) = |X-',F5.2,'|**',F5.2,'.')
      RETURN

C     HEADINGS FOR GROUP 5

    5 GO TO (43,44,41,42),J

C     HEADINGS FOR GROUP 6

    6 WRITE(6,161)M,K1
  161 FORMAT('0','F(X) = FLOOR(',I2,'*X)*(X**',I2,').')
      RETURN

C     HEADINGS FOR GROUP 7

    7 WRITE(6,171)K1,M
  171 FORMAT('0','F(X) = X**',I2,'*SIN(',I2,'*X*PI).')
      RETURN

      END
